c-----------------------------------------------------------------------
c     PRM_HH by A.V.Melnikov (2018)
c
c     The case of bounded phase space (Henon-Heiles system)
c     H = (p_x**2+p_y**2+q_x**2+q_y**2)/2+2*q_x**2*q_y-2/3*q_y**3
c     This code realizes the PRM algorithm.
c     The code contains calls to the dop853 integrator (external).
c-----------------------------------------------------------------------
      program PRM_HH

      implicit none

      integer i,j,k,n_grid

      real*8 t,t_end,y(4),h,eps,rec_time
      real*8 p_1,p_2,q_1,q_2,p_1_2,point(4)
      real*8 q_2_init,q_2_end,q_2_step
      real*8 p_2_init,p_2_end,p_2_step  

      common /inint_point/ point,eps,rec_time

      open(10,file = 'rec_time.dat', status = 'replace')

c    define the value of the Hamiltonian  
      h = 0.1d0   

c    define the radius of the neighborhood where the recurrence is fixed     
      eps = 1.d-3
      
c    define the number of nodes on the q_2 and p_2 axes
      n_grid = 500
 
c    define the location and grid of the initial data
      q_2_init = -0.4d0
      q_2_end = 0.6d0
      q_2_step = (q_2_end-q_2_init)/dble(n_grid)

      p_2_init = -0.5d0
      p_2_end = 0.5d0
      p_2_step = (p_2_end-p_2_init)/dble(n_grid)
      
c     the main loop     
      do i = 0,n_grid
         do j = 0,n_grid

c     t - time  [t,tend] - define the integration time
            t = 0.d0
            t_end = 1.d5
         
c     define the initial data in the plane (q_2, p_2)
            q_1 = 0.d0
            q_2 = q_2_init + q_2_step*dble(i)
            p_2 = p_2_init + p_2_step*dble(j)

c     determine whether the initial data fall into the bounded phase space
            if ((q_2*q_2*(1.d0 - 2.d0*q_2/3.d0) +
     +          p_2*p_2).lt.2.d0*h) then

c     determine the initial value of p_1 from the values of h, q_1 and p_2     
              p_1_2 = 2.d0*(h + q_2*q_2*q_2/3.d0) - p_2*p_2 - q_2*q_2

              if (p_1_2.ge.0.d0) then
                 p_1 = dsqrt(p_1_2)
                 
c     define the initial data in the current grid node
c     in whose neighborhood we are looking for a recurrence
                 y(1) = p_1
                 y(2) = p_2
                 y(3) = q_1
                 y(4) = q_2

                 point = y

c     print a dummy recurrence time for the case when the recurrence 
c     cannot be determined (i.e., the integration time is too small)
                 rec_time = t_end + 1.d0

c     call the subroutine to fix a recurrence
                 call calc_rec_time(t,t_end,y)

                 write(10,'(2f14.7,f14.1)') p_2,q_2,rec_time
              else
c     print a dummy recurrence value for the case when the initial data
c     do not fall into the bounded phase space
                 write(10,'(2f14.7,f14.1)') p_2,q_2,-1.d0
              end if
            else
c     print a dummy recurrence value for the case when the initial data
c     do not fall into the bounded phase space
                 write(10,'(2f14.7,f14.1)') p_2,q_2,-1.d0
            end if

            end do
         end do

      close(10)

      end
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
      subroutine calc_rec_time(t,t_end,y)
c     integrate a trajectory and fix a recurrence

      implicit none
      
      external fcn,solout

      real*8 t,t_end,rpar
      real*8 y(4),work(65),rtol,atol,point(4)

      integer iwork(21),idid,ipar,i

c the integrator parameters (accuracy)
      rtol = 1.d-12
      atol = 1.d-12
      
      do i = 1,65
         work(i) = 0.d0
      end do
      do i = 1,21
         iwork(i) = 0
      end do
      
c    define the maximum step of integration 
      work(6) = 1.d-5

c    determine the maximum number of integration steps  
      iwork(1) = idint((t_end-t)/work(6))
      
      iwork(3) = -1

c call the integrator
      call dop853(4,fcn,t,y,t_end,rtol,atol,0,
     +              solout,1,work,65,iwork,21,
     +              rpar,ipar,idid)

      return
      end
c-----------------------------------------------------------------------
